@@ -1,18 +1,14 @@
 #ifndef S7_H
 #define S7_H
 
-#define S7_VERSION "5.13"
-#define S7_DATE "26-Jan-18"
+#define S7_VERSION "5.16"
+#define S7_DATE "17-Mar-18"
 
 #include <stdint.h>           /* for int64_t */
 
 typedef int64_t s7_int;       /* This sets the size of integers in Scheme; it needs to be big enough to accomodate a C pointer. */
 typedef double s7_double;     /*   similarly for Scheme reals; only double works in C++ */
 
-/* old forms... */
-typedef s7_int s7_Int;
-typedef s7_double s7_Double;
-
 #ifndef __cplusplus
 #ifndef _MSC_VER
   #include <stdbool.h>
@@ -157,7 +153,7 @@ int64_t s7_gc_freed(s7_scheme *sc);
 bool s7_is_eq(s7_pointer a, s7_pointer b);                                   /* (eq? a b) */
 bool s7_is_eqv(s7_pointer a, s7_pointer b);                                  /* (eqv? a b) */
 bool s7_is_equal(s7_scheme *sc, s7_pointer a, s7_pointer b);                 /* (equal? a b) */
-
+bool s7_is_morally_equal(s7_scheme *sc, s7_pointer x, s7_pointer y);         /* (morally-equal? x y) */
 
 bool s7_is_boolean(s7_pointer x);                                            /* (boolean? x) */
 bool s7_boolean(s7_scheme *sc, s7_pointer x);                                /* Scheme boolean -> C bool */
@@ -272,24 +268,16 @@ s7_double s7_real_part(s7_pointer z);                                       /* (
 s7_double s7_imag_part(s7_pointer z);                                       /* (imag-part z) */
 char *s7_number_to_string(s7_scheme *sc, s7_pointer obj, int32_t radix);    /* (number->string obj radix) */
 
-bool s7_is_ulong(s7_pointer arg);
-unsigned long s7_ulong(s7_pointer p);
-s7_pointer s7_make_ulong(s7_scheme *sc, unsigned long n);
-bool s7_is_ulong_long(s7_pointer arg);
-uint64_t s7_ulong_long(s7_pointer p);
-s7_pointer s7_make_ulong_long(s7_scheme *sc, uint64_t n);
-
-
 bool s7_is_vector(s7_pointer p);                                            /* (vector? p) */
 s7_int s7_vector_length(s7_pointer vec);                                    /* (vector-length vec) */
-int32_t s7_vector_rank(s7_pointer vect);                                    /* number of dimensions in vect */
+s7_int s7_vector_rank(s7_pointer vect);                                     /* number of dimensions in vect */
 s7_int *s7_vector_dimensions(s7_pointer vec);                               /* dimensions (don't free the pointer) */
 s7_int *s7_vector_offsets(s7_pointer vec);                                  /* precalculated offsets to speed-up addressing (don't free) */
 s7_pointer *s7_vector_elements(s7_pointer vec);                             /* a pointer to the array of s7_pointers */
 s7_int *s7_int_vector_elements(s7_pointer vec);
 s7_double *s7_float_vector_elements(s7_pointer vec);
-bool s7_is_float_vector(s7_pointer p);                                    
-bool s7_is_int_vector(s7_pointer p);                                      
+bool s7_is_float_vector(s7_pointer p);                                      /* (float-vector? p) */
+bool s7_is_int_vector(s7_pointer p);                                        /* (int-vector? p) */
 
 s7_pointer s7_vector_ref(s7_scheme *sc, s7_pointer vec, s7_int index);                            /* (vector-ref vec index) */
 s7_pointer s7_vector_set(s7_scheme *sc, s7_pointer vec, s7_int index, s7_pointer a);              /* (vector-set! vec index a) */
@@ -334,7 +322,7 @@ s7_pointer s7_hook_set_functions(s7_scheme *sc, s7_pointer hook, s7_pointer func
 bool s7_is_input_port(s7_scheme *sc, s7_pointer p);                         /* (input-port? p) */
 bool s7_is_output_port(s7_scheme *sc, s7_pointer p);                        /* (output-port? p) */
 const char *s7_port_filename(s7_pointer x);                                 /* (port-filename p) */
-int32_t s7_port_line_number(s7_pointer p);                                      /* (port-line-number p) */
+int32_t s7_port_line_number(s7_pointer p);                                  /* (port-line-number p) */
 
 s7_pointer s7_current_input_port(s7_scheme *sc);                            /* (current-input-port) */
 s7_pointer s7_set_current_input_port(s7_scheme *sc, s7_pointer p);          /* (set-current-input-port) */
@@ -553,14 +541,13 @@ s7_pointer s7_apply_function_star(s7_scheme *sc, s7_pointer fnc, s7_pointer args
 s7_pointer s7_call(s7_scheme *sc, s7_pointer func, s7_pointer args);
 s7_pointer s7_call_with_location(s7_scheme *sc, s7_pointer func, s7_pointer args, const char *caller, const char *file, int32_t line);
   
-  /* s7_call takes a Scheme function (e.g. g_car above), and applies it to 'args' (a list of arguments)
-   *   returning the result.
-   *   
+  /* s7_call takes a Scheme function (e.g. g_car above), and applies it to 'args' (a list of arguments) returning the result.
    *   s7_integer(s7_call(s7, g_car, s7_cons(s7, s7_make_integer(s7, 123), s7_nil(s7))));
-   *  
    *   returns 123.
    *
-   * s7_call_with_location passes some information to the error handler.  
+   * s7_call_with_location passes some information to the error handler.
+   * s7_call makes sure some sort of catch exists if an error occurs during the call, but
+   *   s7_apply_function does not -- it assumes the catch has been set up already.
    */
 
 bool s7_is_dilambda(s7_pointer obj);
@@ -818,7 +805,28 @@ s7_pointer s7_apply_n_9(s7_scheme *sc, s7_pointer args,
 #endif
 
 
-#if (!DISABLE_DEPRECATED)
+#if DISABLE_DEPRECATED
+#define s7_is_ulong(arg)          s7_is_integer(arg)
+#define s7_ulong(p)               (uint64_t)s7_integer(p)
+#define s7_make_ulong(sc, n)      s7_make_integer(sc, (s7_int)n)
+
+#define s7_is_ulong_long(arg)     s7_is_c_pointer((void *)arg)
+#define s7_ulong_long(p)          (uint64_t)s7_c_pointer(arg)
+#define s7_make_ulong_long(sc, n) s7_make_c_pointer(sc, (void *)n)
+
+#else
+
+bool s7_is_ulong(s7_pointer arg);
+unsigned long s7_ulong(s7_pointer p);
+s7_pointer s7_make_ulong(s7_scheme *sc, unsigned long n);
+bool s7_is_ulong_long(s7_pointer arg);
+uint64_t s7_ulong_long(s7_pointer p);
+s7_pointer s7_make_ulong_long(s7_scheme *sc, uint64_t n);
+
+/* old forms... */
+typedef s7_int s7_Int;
+typedef s7_double s7_Double;
+
 /* cm uses this: */
 #define s7_UNSPECIFIED(Sc) s7_unspecified(Sc)
 #define s7_NIL(Sc) s7_nil(Sc)
@@ -873,19 +881,20 @@ void s7_object_type_set_direct(int32_t tag,
 void s7_define_function_with_setter(s7_scheme *sc, const char *name, s7_function get_fnc, 
 				    s7_function set_fnc, int32_t req_args, int32_t opt_args, const char *doc);
   /* this is now the same as s7_dilambda (different args) */
-#endif
 
-/* obsolete names, will be deprecated eventually */
 #define s7_procedure_setter        s7_setter
 #define s7_procedure_documentation s7_documentation
 #define s7_procedure_signature     s7_signature
+#endif
 
 
 /* --------------------------------------------------------------------------------
  * 
  *        s7 changes
  *
+ * 17-Mar:    deprecate s7_ulong and s7_ulong_long functions.
  * 26-Jan-18: s7_set_setter.
+ * --------
  * 11-Dec:    s7_gc_protect_via_stack
  * 3-Oct:     renamed procedure-signature -> signature, procedure-documentation -> documentation, and procedure-setter -> setter.
  * 18-Sep:    s7_immutable, s7_is_immutable. define-constant follows lexical scope now.
