@@ -55,11 +55,11 @@
 
 #include "snd-strings.h"
 
-#define SND_DATE "15-Feb-18"
+#define SND_DATE "19-Mar-18"
 #ifndef SND_VERSION
-#define SND_VERSION "18.1"
+#define SND_VERSION "18.2"
 #endif
 #define SND_MAJOR_VERSION "18"
-#define SND_MINOR_VERSION "1"
+#define SND_MINOR_VERSION "2"
 
 #endif
