@@ -2,25 +2,86 @@
 
 ;(set! (hook-functions *load-hook*) (list (lambda (hook) (format () "loading ~S...~%" (hook 'name)))))
 
+(define (cycler size)
+  (let ((cp-lst (make-list 3 #f))
+	(it-lst (make-list 3 #f)))
+    (let ((bases (vector (make-list 3 #f)
+			 (make-vector 3 #f)
+			 (hash-table* 'a 1 'b 2 'c 3)
+			 (inlet 'a 1 'b 2 'c 3)
+			 (make-iterator it-lst)
+			 (c-pointer 1 cp-lst)))
+	  (sets ()))
+	  
+      (do ((blen (length bases))
+	   (i 0 (+ i 1)))
+	  ((= i size))
+	(let ((r1 (random blen))
+	      (r2 (random blen))
+	      (loc (random 3)))
+	  (let ((b1 (bases r1))
+		(b2 (bases r2)))
+	    (case (type-of b1)
+	      ((pair?)
+	       (if (> (random 10) 3)
+		   (begin
+		     (set! (b1 loc) b2)
+		     (set! sets (cons (list r1 loc r2) sets)))
+		   (begin
+		     (set-cdr! (list-tail b1 2) (case loc ((0) b1) ((1) (cdr b1)) (else (cddr b1))))
+		     (set! sets (cons (list r1 (+ loc 3) r2) sets)))))
+
+	      ((vector?)
+	       (set! (b1 loc) b2)
+	       (set! sets (cons (list r1 loc r2) sets)))
+
+	      ((hash-table? let?)
+	       (let ((key (#(a b c) loc)))
+		 (set! (b1 key) b2)
+		 (set! sets (cons (list r1 key r2) sets))))
+
+	      ((c-pointer?)
+	       (set! (cp-lst loc) b2)
+	       (set! sets (cons (list r1 loc r2) sets)))
+		
+	      ((iterator?)
+	       (set! (it-lst loc) b2)
+	       (set! sets (cons (list r1 loc r2) sets)))))))
+
+      (object->string (bases (random 6)) :readable))))
+
 (load "stuff.scm")
 ;(load "write.scm")
-;(load "mockery.scm")
+(load "mockery.scm")
 
-;(set! (*s7* 'safety) 2)
+(define mock-number (*mock-number* 'mock-number))
+(define mock-pair (*mock-pair* 'mock-pair))
+(define mock-string (*mock-string* 'mock-string))
+(define mock-char (*mock-char* 'mock-char))
+(define mock-vector (*mock-vector* 'mock-vector))
+(define mock-symbol (*mock-symbol* 'mock-symbol))
+(define mock-hash-table* (*mock-hash-table* 'mock-hash-table*))
+
+(set! (*s7* 'safety) 1)
 
 (set! (*s7* 'max-stack-size) 32768)
 (set! (*s7* 'max-heap-size) (ash 1 23)) ; 8M -- 560000000 is about 8G
 ;(set! (*s7* 'gc-stats) 6)
 (set! (*s7* 'print-length) 1000)
-(define ostr "")
-(define nostr "")
 (set! (*s7* 'max-string-length) 100000)
 (set! (*s7* 'max-list-length) 10000)
 (set! (*s7* 'max-vector-length) 10000)
 (set! (*s7* 'max-vector-dimensions) 10)
 (set! (*s7* 'autoloading?) #f)
+(set! (*s7* 'morally-equal-float-epsilon) 1.0e-6)
 (set! (current-output-port) #f)
+(define startup-input-port (current-input-port))
+
+(define ostr "")
+(define nostr "")
+
 (define __var__ #f)
+(define __old_var__ #f)
 (define __var1__ #f)
 (define error-type #f)
 (define error-info #f)
@@ -34,6 +95,10 @@
 (define __var2__ 3)
 (set! (symbol-setter '__var2__) (lambda (s v) (if (integer? v) v 3)))
 
+(define (free1) (set! x (- (+ x 1) 1)))
+(define (free2) (x i))
+(define (free3) (local-func 0))
+(define (_vals_) (values #f 1 2))
 (define (finite? n) (not (or (nan? n) (infinite? n))))
 
 (define (s7-print-length) (*s7* 'print-length))
@@ -131,8 +196,8 @@
 (define (s7-free-heap-size) (*s7* 'free-heap-size))
 (define (s7-gc-freed) (*s7* 'gc-freed))
 (define (s7-gc-protected-objects) (*s7* 'gc-protected-objects))
-(define (s7-memory-usage) (*s7* 'memory-usage))
 |#
+(define (s7-memory-usage) (string? (with-output-to-string (lambda () (*s7* 'memory-usage)))))
 (define (s7-history) (*s7* 'history))
 (define (s7-history-size) (*s7* 'history-size))
 
@@ -182,6 +247,23 @@
   (and (null? (cyclic-sequences code))
        (eval code)))
 
+(define (checked-hash-table* . args)
+  (let ((_h_ (make-hash-table (*s7* 'default-hash-table-length) morally-equal?)))
+    (do ((key/value args (cddr key/value)))
+	((null? key/value) _h_)
+      (if (not (pair? (cdr key/value)))
+	  (error 'wrong-number-of-args "no value")
+	  (set! (_h_ (car key/value)) (cadr key/value))))))
+
+(define (checked-hash-table . args)
+  (let ((_h_ (make-hash-table (*s7* 'default-hash-table-length) morally-equal?)))
+    (do ((key/value args (cdr key/value)))
+	((null? key/value) _h_)
+      (if (not (pair? key/value))
+	  (error 'wrong-type-args "not a pair")
+	  (set! (_h_ (car key/value)) (cdr key/value))))))
+
+
 (load "s7test-block.so" (sublet (curlet) (cons 'init_func 'block_init)))
 
 (define-expansion (_dw_ . args)
@@ -236,7 +318,13 @@
   `(let ((_f_ (lambda () ,@args))) (_f_) (_f_)))
 
 (define-expansion (_ft2_ . args)
-  `(let () (define (_f_) ,@args) (_f_) (_f_)))
+  `(let () (define (_f_) ,@args) (define (g) (_f_)) (g) (g)))
+
+(define-expansion (_fa1_ . args)
+  `(let ((_y_ (begin ,@args))) (define (f x) (x)) (f _y_)))
+
+(define-expansion (_fa2_ . args)
+  `((lambda (x) (apply x ())) (begin ,@args)))
 
 (define-expansion (_lt2_ . args)
   `(let ((mx max)) ((lambda* ((max min) (min mx)) ,@args))))
@@ -253,26 +341,60 @@
        (do ((i 0 (+ i 1)))
 	   ((= i 1))
 	 ,@(map (lambda (x)
-		  `(display ,x))
+		  (list 'display x))
 		args)))))
 
 (define-expansion (_do2_ . args)
   `(with-output-to-string 
      (lambda () 
        ,@(map (lambda (x)
-		`(display ,x))
+		(list 'display x))
 	      args))))
 
 (define-expansion (_do3_ . args)
   `(let ((exiter (vector #f))) (do ,(car args) ((vector-ref exiter 0) 1) ,@(cdr args) (vector-set! exiter 0 #t))))
 
+(define-expansion (_cop1_ . args)
+  `(let ((x begin ,@args))
+     x))
+
+(define-expansion (_cop2_ . args)
+  `(let ((x begin ,@args))
+     (copy x)))
+
+(define-expansion (_rd1_ . args)
+  `(let ((port #f))
+     (dynamic-wind
+	 (lambda ()
+	   (set! port (open-input-string (format #f "~{~W~^ ~}" (list ,@args)))))
+	 (lambda ()
+	   (read port))
+	 (lambda ()
+	   (close-input-port port)))))
+
+(define-expansion (_rd2_ . args)
+  `(with-input-from-string
+       (object->string (car (list ,@args)) :readable)
+     read))
+
+
+(define ims (immutable! (string #\a #\b #\c)))
+(define imbv (immutable! (byte-vector 0 1 2)))
+(define imv (immutable! (vector 0 1 2)))
+(define imiv (immutable! (int-vector 0 1 2)))
+(define imfv (immutable! (float-vector 0 1 2)))
+(define imi (immutable! (inlet 'a 1 'b 2)))
+(define imb (immutable! (block 0.0 1.0 2.0)))
+(define imh (immutable! (hash-table* 'a 1 'b 2)))
+;(define imp (immutable! (cons 0 (immutable! (cons 1 (immutable! (cons 2 ())))))))
 
 (set! (hook-functions *unbound-variable-hook*) ())
 (define x 0)
 (define max-stack (*s7* 'stack-top))
+(define last-error-type #f)
 
 (let ((functions (vector 'not '= '+ 'cdr 'real? 'rational? 'number? '> '- 'integer? 'apply 
-			    'catch 'length 'eq? 'car '< 'assq 'complex? 'vector-ref 
+			  'catch 'length 'eq? 'car '< 'assq 'complex? 'vector-ref 
 			  'abs '* 'null? 'imag-part '/ 'vector-set! 'equal? 'magnitude 'real-part 'pair? 'max 'nan? 'string->number 'list
 			  'negative? 'cons 'string-set! 'list-ref 'eqv? 'positive? '>= 'expt 'number->string 'zero? 'floor 'denominator 'integer->char 
 			  'string? 'min '<= 'char->integer 'cos 'rationalize 'cadr 'sin 'char=? 'map 'list-set! 'defined? 'memq 'string-ref 'log 
@@ -313,7 +435,9 @@
 			  'call-with-input-string 'documentation 
 			  'continuation? 'hash-table? 'port-closed? 
 			  'output-port? 'input-port? 
-			  'provide 'call-with-output-string 'hash-table 
+			  'provide 'call-with-output-string 
+			  ;; 'hash-table 
+			  ;; 'checked-hash-table 'checked-hash-table*
 			  ;;'current-output-port 
 			  'with-output-to-string 
 			  ;;'current-input-port -- too many (read...)
@@ -325,7 +449,9 @@
 			  ;; -------- naming funcs
 			  'make-hook 
 			  'let 'let* 'letrec 
+#|
 			  'lambda 'lambda*
+|#
 			  'multiple-value-bind 'call-with-values
 			  'inlet 
 			  'object->let
@@ -388,7 +514,9 @@
 			  '*autoload*
 			  ;;'*error-hook*
 
-			  'sequence? 'directory? 'hash-table-entries 'hash-table* 'arity 'logbit? 
+			  'sequence? 'directory? 'hash-table-entries 
+			  ;;'hash-table* -- handled as morally equal
+			  'arity 'logbit? 
 			  'random-state? 'throw 'float-vector-set! 'make-iterator 'complex 
 			  'let-ref 'int-vector 'aritable? 'gensym? 'syntax? 'iterator-at-end? 'let? 
 			  'make-shared-vector 'float-vector 'iterator-sequence 'getenv 'float-vector-ref 
@@ -403,7 +531,9 @@
 			  'byte-vector-ref 'file-exists? 'make-int-vector 'string-downcase 'string-upcase 
 			  'byte-vector 'morally-equal? 
 			  ;;'let-set! -- rootlet troubles?
-			  'c-pointer? 'int-vector-ref 'coverlet 'float? 
+			  'c-pointer? 'int-vector-ref ;
+			  ;;'coverlet -- blocks block's morally-equal?
+			  'float? 
 			  'list-values 'byte-vector? 'openlet? 'iterator? 
 			  'string->byte-vector
 
@@ -432,7 +562,8 @@
 			  ;;'s7-set-bignum-precision 
 			  ;;'s7-set-float-format-precision
 |#
-			  's7-memory-usage 
+			  's7-memory-usage
+			  'undefined-function
 			  ;;'s7-safety 's7-set-safety ;-- these need work...
 			  's7-c-types 
 			  ;;'s7-history ;-- causes stack to grow?
@@ -448,7 +579,8 @@
 			  's7-rootlet-size 's7-heap-size 's7-free-heap-size 's7-gc-freed 's7-stack-size 's7-max-stack-size 's7-gc-stats
 			  's7-stacktrace-defaults
 
-			  'macroexpand 'block-reverse! 'subblock 'local-symbol? 'unquote 'block-append 'block-let
+			  ;'macroexpand ;-- uninteresting objstr stuff
+			  'block-reverse! 'subblock 'unquote 'block-append ;'block-let
 
 			  ;'subsequence 
 			  'empty? 'indexable? 'first
@@ -462,17 +594,18 @@
 			  'log-n-of ; stack grows if n < 0?
 			  'pp
 
-			  'kar '_dilambda_
+			  'kar '_dilambda_ '_vals_
+			  'free1 'free2 'free3
 
 			  'tree-cyclic?
 			  ))
 	 
-      (args (vector "-123" "1234" "-3/4" "-1" "(expt 2 32)" "4294967297" "(+ a 1)" "(- a 1)" "(logand (ash 1 b) a)"
+      (args (vector "-123" "1234" "-3/4" "-1" "(expt 2 32)" "4294967297" "1001" "10001" "(+ a 1)" "(- a 1)" "(logand (ash 1 b) a)"
 		    "(make-block 2)" "(block 1.0 2.0 3.0)" "(block)"
 		    "\"ho\"" ":ho" "'ho" "':go" "(list 1)" "(list 1 2)" "(cons 1 2)" "'()" "(list (list 1 2))" "(list (list 1))" "(list ())" "=>" 
 		    "#f" "#t" "()" "#()" "\"\"" "'#()" ":readable" ":rest" ":allow-other-keys" ":a" ;"__func__"
 		    "1/0+i" "0+0/0i" "0+1/0i" "1+0/0i" "0/0+0/0i" "0/0+i" 
-		    "cons" "''2" "\"ra\""
+		    "cons" "''2" "\"ra\"" "gad1.data"
 		    "(make-hook)" "(make-hook '__x__)"
 		    "1+i" "0+i" "(ash 1 43)" 
 		    "(integer->char 255)" "(string (integer->char 255))" "(string #\\null)" "(byte-vector 0)"
@@ -484,7 +617,7 @@
 		    "#(123 223)" "(vector 1 '(3))" "(let ((x 3)) (lambda (y) (+ x y)))" "abs" "(lambda sym-args sym-args)" "#u8(0 1)"
 		    "(dilambda (lambda () 1) (lambda (a) a))" "quasiquote" "macroexpand" "(lambda* ((a 1) (b 2)) (+ a b))" 
 		    "((lambda (a) (+ a 1)) 2)" "((lambda* ((a 1)) (+ a 1)) 1)" "(lambda (a) (values a (+ a 1)))" "((lambda (a) (values a (+ a 1))) 2)"
-		    "(define-macro (_m1_ a) `(+ ,a 1))"
+		    "(define-macro (_m1_ a) `(+ ,a 1))" "(define-bacro (_b1_ a) `(* ,a 2))"
 		    "(string #\\c #\\null #\\b)" "#2d((100 200) (3 4))" "#r(0 1)" "#i2d((101 201) (3 4))" "#r2d((.1 .2) (.3 .4))" "#i1d(15 25)"
 		    "(values 1 2)" "(values)" "(values #\\c 3 1.2)" "(values \"ho\")"
 		    "`(x)" "`(+ x 1)" "`(x 1)" "`((x))" "`((+ x 1))" "`(((+ x 1)))" "`((set! x (+ x 1)) (* x 2))" "`((x 1))" "`(((x 1))) "
@@ -498,23 +631,25 @@
 		    "((i 0 (+ i 1)))" "(null? i)" 
 		    "(= i 2)" "(zero? i)" "((null? i) i)" "(#t ())" 
 		    "(x => y)" "((0 1) ())" "(- i 1)" "(if x y)" "(or x y)"
-		    ;;"(begin (f x) B)" "(A (f x) B)" 
 		    ;;"(f x) i" "x y z" "1 2"
 		    "`(+ ,a ,@b)" "`(+ ,a ,b)" "`(+ ,a ,b ,@c)" "`(+ ,a b ,@c ',d)"
 		    "_definee_" "(_definee_ __var__)" "(_definee_ x)" 
 		    "(hash-table* 'a 1)" "(hash-table)" 
 		    "(make-iterator (list 11 22 33))" "(make-iterator (vector 1 2 3))" "(make-iterator (string #\\1))" "(make-iterator x)" 
 		    "#<eof>" "#<undefined>" "#<unspecified>"
-		    "#o123" "#b101" "#\\newline" "#_cons" "#x123.123" "#\\x65" ;"_1234_" "kar"
+		    "#o123" "#b101" "#\\newline" "#\\alarm" "#\\delete" "#_cons" "#x123.123" "#\\x65" ;"_1234_" "kar"
 		    
 		    "(call-with-exit (lambda (goto) goto))"
 		    "(with-baffle (call/cc (lambda (cc) cc)))"
-		    "(symbol->string 'x)" "(symbol \"a b\")" "(symbol \"(\")\")"
-		    "(setter _definee_)" "(setter car)" "(setter kar)"
+		    "(symbol->string 'x)" "(symbol \"a b\")" "(symbol \"(\\\")\")"
+		    ;;"(setter _definee_)"
+		    "(setter car)" "(setter kar)"
 		    "(call-with-exit (lambda (return) (let ((x 1) (y 2)) (return x y))))"
 		    "(call/cc (lambda (return) (let ((x 1) (y 2)) (return x y))))"
 		    "(let ((x 1)) (dynamic-wind (lambda () (set! x 2)) (lambda () (+ x 1)) (lambda () (set! x 1))))"
 
+		    "(let ((x 1)) (free1))" "(let ((x #(1)) (i 0)) (free2))" "(let () (define (local-func x) x) (free3))"
+
 		    "1+1e10i" "1e15+1e15i" "0+1e18i" "1e18" 
 		    ;;"(real-part (random 0+i))" -- (cond (real-part...))!
 		    ;;"(random 1.0)" ; number->string so lengths differ
@@ -522,18 +657,27 @@
 		    ;;"(else ())" "(else (f x) B)"
 		    "(else)"
 		    "else" "x" "(+ x 1)" "(+ 1/2 x)" "(abs x)" "(+ x 1 2+i)" "(* 2 x 3.0 4)" "((x 1234))" "((x 1234) (y 1/2))" "'x" "(x 1)"
-		    "_undef_"
+		    "_undef_" "(begin |undef1|)"
 		    "+signature+" "+documentation+" "+setter+" 
 		    "__var2__"
-		    "\"~S~%\"" "\"~A~D~X\"" "\"~{~A~^~}~%\"" "\"~NC~&\""
-		    "(immutable! (string #\\a #\\b #\\c))" "(immutable! (byte-vector 0 1 2))" 
-		    "(immutable! (vector 0 1 2))" "(immutable! (int-vector 0 1 2))" "(immutable! (float-vector 0 1 2))" 
-		    "(immutable! (inlet 'a 1 'b 2))" 
-		    "(immutable! (block 0.0 1.0 2.0))" 
-		    "(immutable! (hash-table* 'a 1 'b 2))"
-		    "(immutable! (cons 0 (immutable! (cons 1 (immutable! (cons 2 ()))))))"
+		    ;; "\"~S~%\"" "\"~A~D~X\"" "\"~{~A~^~}~%\"" "\"~NC~&\"" -- creates files by these names!
+		    "ims" "imbv" "imv" "imiv" "imfv" "imi" "imb" "imh" ;;"imp"--many ways to cloober this
+		    ;"(immutable! (string #\\a #\\b #\\c))" "(immutable! (byte-vector 0 1 2))" 
+		    ;"(immutable! (vector 0 1 2))" "(immutable! (int-vector 0 1 2))" "(immutable! (float-vector 0 1 2))" 
+		    ;"(immutable! (inlet 'a 1 'b 2))" 
+		    ;"(immutable! (block 0.0 1.0 2.0))" 
+		    ;"(immutable! (hash-table* 'a 1 'b 2))"
+		    ;"(immutable! (cons 0 (immutable! (cons 1 (immutable! (cons 2 ()))))))"
 		    ;"(immutable! 'x)"
 
+		    "(mock-number 0)" "(mock-number 1-i)" "(mock-number 4/3)" "(mock-number 2.0)"
+		    "(mock-string #\\h #\\o #\\h #\\o)"
+		    "(mock-pair '(2 3 4))"
+		    "(mock-char #\\b)"
+		    "(mock-symbol 'c)"
+		    "(mock-vector 1 2 3 4)"
+		    "(mock-hash-table* 'b 2)"
+
 		    ;;"(make-list 16 0)" "(make-vector 16 0)" "(make-int-vector 16 0)" "(make-float-vector 16 0)" "(make-byte-vector 16 0)"
 		    ;;"(make-string 16 #\\0)"
 		    ;;"(let ((hash (make-hash-table))) (do ((i 0 (+ i 1))) ((= i 16) hash) (hash-table-set! hash i 0)))"
@@ -545,15 +689,24 @@
 		    "(append (block) (block))"
 		    "(let-temporarily ((x 1234)) (+ x 1))"
 		    "(error 'oops \"an error!\")"
-		    "(set! (symbol-setter 'x) (lambda (s v) 1))"
 
-		    "(catch #t 1 cons)" "(catch #t (lambda () (fill! (rootlet) 1)) (lambda (type info) info))"
+		    ;;"(catch #t 1 (lambda (a b) b))" "(catch #t (lambda () (fill! (rootlet) 1)) (lambda (type info) info))"
 
 		    "#xfeedback" "#_asdf"
 		    ;"quote" "'"
 		    "if" "begin" "cond" "case" "when" "unless" "letrec" "letrec*" "or" "and" "let-temporarily"
 		    "lambda*" "lambda"
-		    ;; "let" "let*" "do" "set!" "with-let" "define" "define*" "define-macro" "define-macro*" "define-bacro" "define-bacro*"
+		    ;;"let" "let*" "do" "set!" "with-let" ;"define" "define*" "define-macro" "define-macro*" "define-bacro" "define-bacro*"
+
+		    "(begin (string? (stacktrace)))"
+
+		    "(let ((<1> (vector #f))) (set! (<1> 0) <1>) <1>)"
+		    "(let ((<1> (inlet :a #f))) (set! (<1> :a) <1>) <1>)"
+		    "(let ((<1> (hash-table*))) (set! (<1> 'a) <1>) <1>)"
+		    "(let ((<1> #f) (<2> (vector #f))) (set! <1> (make-iterator <2>)) (set! (<2> 0) <1>) <1>)"
+		    "(let ((<1> (list 1 #f))) (set! (<1> 1) (let ((<L> (list #f 3))) (set-car! <L> <1>) <L>)) <1>)"
+
+		    #f #f
 		    ))
 
       (codes (vector 
@@ -578,12 +731,13 @@
 	      ;;(list "((define-macro (_m_) " "((define-bacro (_m_) ") ; circular source if signature in body
 	      ;;(list "(let ((mx max)) (let ((max min) (min mx)) " "(begin (_lt2_ ") ; loops?
 	      ;;(list "(begin (letrec ((x 1)) " "(begin (letrec* ((x 1)) ")
-	      (reader-cond ((not (provided? 'pure-s7)) (list "(with-input-from-string \"1234\" (lambda () " "(begin (_dw_string_ ")))
+	      ;(reader-cond ((not (provided? 'pure-s7)) (list "(with-input-from-string \"1234\" (lambda () " "(begin (_dw_string_ ")))
 	      (list "(map abs (begin " "(map (lambda (x) (if (>= x 0.0) x (- x))) (begin ")
 	      (list "(for-each display (list " "(for-each (lambda (x) (display x)) (list ")
 	      (list "(begin (_ct1_ " "(begin (_ct2_ ")
 	      (list "(begin (_mem1_ " "(begin (_mem2_ ")
 	      (list "(begin (_ft1_ " "(begin (_ft2_ ")
+	      (list "(begin (_fa1_ " "(begin (_fa2_ ")
 	      (list "(with-output-to-string (lambda () " "(begin (_dw_out_ ")
 	      (list "(begin (_rf1_ " "(begin (_rf2_ ")
 	      (list "(let () (_do1_ " "(let () (_do2_ ")
@@ -592,6 +746,8 @@
 	      (list "(let ((x 1)) (immutable! 'x) (begin " "((lambda* ((x 1)) (immutable! 'x) ")
 	      (list "(do ((i 0 (+ i 1))) ((= i 1)) (do ((j 0 (+ j 1))) ((= j 1)) "
 		    "(do ((i 0 (+ i 1))) ((= i 1)) (let ((j 0)) ")
+	      (list "(or (_cop1_ " "(and (_cop2_ ")
+	      (list "(values (_rd1_ " "(append (_rd2_ ")
 	      ))
       
       (chars (vector #\( #\( #\) #\space))) ; #\/ #\# #\, #\` #\@ #\. #\:))  ; #\\ #\> #\space))
@@ -600,6 +756,12 @@
 	(alen (length args))
 	(codes-len (length codes)))
 
+    (define (get-arg)
+      (let ((str (args (random alen))))
+	(if (string? str)
+	    str
+	    (cycler 4))))
+
     ;(for-each (lambda (x) (if (not (symbol? x)) (format *stderr* "~A " x))) functions)
     ;(for-each (lambda (x) (if (not (string? x)) (format *stderr* "~A " x))) args)
     ;(do ((p (vector->list functions) (cdr p))) ((null? p)) (if (memq (car p) (cdr p)) (format *stderr* "~A repeats~%" (car p))))
@@ -630,8 +792,8 @@
 	  (set! j 1)
 	  (fill! str #\space)
 	  (set! (str 0) #\()
-	  (let* ((op (functions (random flen)))
-		 (opstr (fix-op op)))
+
+	  (let ((opstr (fix-op (functions (random flen)))))
 	    (do ((oplen (length opstr))
 		 (n 0 (+ n 1))
 		 (k j (+ k 1)))
@@ -656,8 +818,7 @@
 		(case (str j)
 		  ((#\()
 		   (set! parens (+ parens 1))
-		   (let* ((op (functions (random flen)))
-			  (opstr (fix-op op)))
+		   (let ((opstr (fix-op (functions (random flen)))))
 		     (do ((oplen (length opstr))
 			  (n 0 (+ n 1))
 			  (k (+ j 1) (+ k 1)))
@@ -677,7 +838,7 @@
 		   (let ((nargs (random 5)))
 		     (do ((n 0 (+ n 1)))
 			 ((= n nargs))
-		       (let ((argstr (args (random alen))))
+		       (let ((argstr (get-arg)))
 			 (do ((arglen (length argstr))
 			      (n 0 (+ n 1))
 			      (k (+ j 1) (+ k 1)))
@@ -711,52 +872,78 @@
       (if (and (eq? (type-of val1) (type-of val2))
 	       (eq? (type-of val1) (type-of val3))
 	       (eq? (type-of val1) (type-of val4)))
-	  (unless (or (openlet? val1)
+	  (cond ((or (openlet? val1)
 		      (string-position "(set!" str1)
-		      (string-position "gensym" str1))
+		      (string-position "gensym" str1)))
 
-	    (cond ((or (and (symbol? val1)
-			    (not (gensym? val1)))
-		       (boolean? val1)
-		       (syntax? val1))
-		   (unless (and (eq? val1 val2)
-				(eq? val1 val3)
-				(eq? val1 val4))
-		     (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%   ~S ~S ~S ~S~%" 
-			     str str1 str2 str3 str4 
-			     val1 val2 val3 val4)))
-
-		  ((sequence? val1)
-		   (let ((len1 (length val1)))
-		     (unless (or (let? val1)
-				 (and (eqv? len1 (length val2))
-				      (eqv? len1 (length val3))
-				      (eqv? len1 (length val4))))
-		       (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%    ~S~%    ~S~%    ~S~%    ~S~%~%" 
+		((symbol? val1)
+		 (if (gensym? val1)
+		     (unless (and (gensym? val2)
+				  (gensym? val3)
+				  (gensym? val4))
+		       (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%   ~S ~S ~S ~S~%" 
 			       str str1 str2 str3 str4 
 			       val1 val2 val3 val4))
-		     (if (or (and (string? val1)
-				  (not (and (eq? (byte-vector? val1) (byte-vector? val2))
-					    (eq? (byte-vector? val1) (byte-vector? val3))
-					    (eq? (byte-vector? val1) (byte-vector? val4)))))
-			     (and (gensym? val1)
-				  (not (and (gensym? val2) (gensym? val3) (gensym? val4))))
-			     (and (keyword? val1)
-				  (not (and (keyword? val2) (keyword? val3) (keyword? val4)))))
-			 (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%    ~S~%    ~S~%    ~S~%    ~S~%~%" 
-				 str str1 str2 str3 str4 
-				 val1 val2 val3 val4))))
-
-		  ((number? val1)
-		   (if (or (and (nan? val1)
-				(not (and (nan? val2) (nan? val3) (nan? val4))))
-			   (and (infinite? val1)
-				(not (and (infinite? val2) (infinite? val3) (infinite? val4))))
-			   (and (finite? val1)
-				(not (and (finite? val2) (finite? val3) (finite? val4)))))
+		     (unless (and (eq? val1 val2)
+				  (eq? val1 val3)
+				  (eq? val1 val4))
+		       (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%   ~S ~S ~S ~S~%" 
+			       str str1 str2 str3 str4 
+			       val1 val2 val3 val4))))
+		
+		((sequence? val1)
+		 (let ((len1 (length val1)))
+		   (unless (or (let? val1)
+			       (and (eqv? len1 (length val2))
+				    (eqv? len1 (length val3))
+				    (eqv? len1 (length val4))))
+		     (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%    ~S~%    ~S~%    ~S~%    ~S~%~%" 
+			     str str1 str2 str3 str4 
+			     val1 val2 val3 val4))
+		   (if (and (string? val1)
+			    (not (and (eq? (byte-vector? val1) (byte-vector? val2))
+				      (eq? (byte-vector? val1) (byte-vector? val3))
+				      (eq? (byte-vector? val1) (byte-vector? val4)))))
 		       (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%    ~S~%    ~S~%    ~S~%    ~S~%~%" 
 			       str str1 str2 str3 str4 
-			       val1 val2 val3 val4)))))
+			       val1 val2 val3 val4))))
+		
+		((number? val1)
+		 (if (or (and (nan? val1)
+			      (not (and (nan? val2) (nan? val3) (nan? val4))))
+			 (and (infinite? val1)
+			      (not (and (infinite? val2) (infinite? val3) (infinite? val4))))
+			 (and (finite? val1)
+			      (not (and (finite? val2) (finite? val3) (finite? val4))))
+			 (and (real? val1) (real? val2) (real? val3) (real? val4) 
+			      (or (and (negative? val1) (or (positive? val2) (positive? val3) (positive? val4)))
+				  (and (positive? val1) (or (negative? val2) (negative? val3) (negative? val4))))))
+		     (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%    ~S~%    ~S~%    ~S~%    ~S~%~%" 
+			     str str1 str2 str3 str4 
+			     val1 val2 val3 val4)))
+		
+		((or (boolean? val1)
+		     (syntax? val1)
+		     (unspecified? val1)
+		     (char? val1)
+		     (eof-object? val1)
+		     (null? val1))
+		 (unless (and (eq? val1 val2)
+			      (eq? val1 val3)
+			      (eq? val1 val4))
+		   (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%   ~S ~S ~S ~S~%" 
+			   str str1 str2 str3 str4 
+			   val1 val2 val3 val4)))
+		
+		((or (undefined? val1)
+		     (c-object? val1))
+		 (unless (and (equal? val1 val2)
+			      (equal? val1 val3)
+			      (equal? val1 val4))
+		   (format *stderr* "~%~%~S~%~S~%~S~%~S~%~S~%   ~S ~S ~S ~S~%" 
+			   str str1 str2 str3 str4 
+			   val1 val2 val3 val4)))
+		)
 	  (begin
 	    (format *stderr* "~%~%~S~%~S~%~S~%~S~%    ~S~%    ~S~%    ~S~%    ~S~%" 
 		    str1 str2 str3 str4 
@@ -772,16 +959,27 @@
 
     (define (eval-it str) ;(format #t "~A~%" str)
       ;(format *stderr* "~S~%" str)
+      (set! __var__ __old_var__)
       (catch #t 
 	(lambda ()
 	  (car (list (eval-string str))))
 	(lambda (type info)
 	  (set! error-type type)
 	  (set! error-info info)
+	  (when (and last-error-type
+		     (not (eq? error-type last-error-type)))
+	    (format *stderr* "~S ~S~%" last-error-type error-type)
+	    (set! last-error-type error-type))
+	  (if (eq? type 'stack-too-big)
+	      (format *stderr* "stack overflow from ~S~%" str))
+	  (when (eq? type 'heap-too-big)
+	    (format *stderr* "heap overflow from ~S~%" str)
+	    (abort))
           'error)))
 
     (define (try-both str)
       (set! ostr str)
+
       (catch #t 
 	(lambda () 
 	  (s7-optimize (list (catch #t 
@@ -789,6 +987,8 @@
 				 (with-input-from-string str read))
 			       (lambda args ())))))
 	(lambda arg 'error))
+
+      (set! last-error-type #f)
       (let* ((outer (codes (random codes-len)))	     
 	     (str1 (string-append "(let ((x #f) (i 0)) " (car outer) str ")))"))
 	     (str2 (string-append "(let () (define (func) " str1 ") (define (hi) (func)) (hi))"))
@@ -798,8 +998,50 @@
 	      (val2 (eval-it str2))
 	      (val3 (eval-it str3))
 	      (val4 (eval-it str4)))
-	  (same-type? val1 val2 val3 val4 str str1 str2 str3 str4)))
-      (let ((nstr (make-expr (+ 1 (random 6)))))
+	  (same-type? val1 val2 val3 val4 str str1 str2 str3 str4)
+
+	  (when (current-output-port)
+	    (format *stderr* "current-output-port is ~S from ~S and ~S~%" (current-output-port) str1 str2)
+	    (set! (current-output-port) #f))
+
+	  (unless (eq? (current-input-port) startup-input-port)
+	    (format *stderr* "current-input-port is ~S from ~S and ~S~%" (current-input-port) str1 str2)
+	    (set! (current-input-port) startup-input-port))
+
+	  ;; if val1 examined, remember: (if (and (let? val1) (openlet? val1)); (not (eq? val1 (rootlet)))) (coverlet val1)) ; might be open and have a length func
+	  ))
+
+#|
+      (catch #t
+	(lambda ()
+	  (let ((val1 (car (list (eval-string str)))))
+	    (let ((newstr (object->string val1 :readable)))
+	      (let ((val2 (eval-string newstr)))
+		(if (not (morally-equal? val1 val2))
+		    (format *stderr* "~%~S~%~S~%    ~W -> ~W~%" str newstr val1 val2))))))
+	(lambda arg 'error))
+
+      (catch #t
+	(lambda ()
+	  (let ((val1 (list (eval-string str))))
+	    (let ((val2 (list (eval (with-input-from-string str read)))))
+	      (call-with-exit 
+	       (lambda (ok)
+		 (for-each
+		  (lambda (x y)
+		    (unless (and (morally-equal x y)
+				 (not (gensym? x)))
+		      (format *stderr* "eval/string ~%~S~%    ~W -> ~W~%" str val1 val2)
+		      (ok)))
+		  val1 val2))))))
+	(lambda arg 'error))
+
+      (catch #t
+	(lambda ()
+	  (pretty-print (with-input-from-string str read)))
+	(lambda arg 'error))
+
+      (let ((nstr (make-expr (+ 1 (random 4)))))
 	(set! nostr nstr)
 	(catch #t 
 	  (lambda () 
@@ -816,7 +1058,29 @@
 		(val6 (eval-it str6))
 		(val7 (eval-it str7))
 		(val8 (eval-it str8)))
-	    (same-type? val5 val6 val7 val8 nstr str5 str6 str7 str8)))))
+	    (same-type? val5 val6 val7 val8 nstr str5 str6 str7 str8)))
+	(let ((str5 (string-append "(let ((x 0)) (cond ((not ((lambda args (car args)) (let () " nstr "))) #<unspecified>) (else " str ")))"))
+	      (str6 (string-append "(let ((x 0)) (when (let () " nstr ") " str "))"))
+	      (str7 (string-append "(let ((x 0)) (unless (not ((lambda args (car args)) (let () " nstr "))) " str "))"))
+	      (str8 (string-append "(let ((x 0)) (cond ((let () " nstr ") " str ")))")))
+	  (let ((val5 (eval-it str5))
+		(val6 (eval-it str6))
+		(val7 (eval-it str7))
+		(val8 (eval-it str8)))
+	    (same-type? val5 val6 val7 val8 nstr str5 str6 str7 str8))))
+
+	(let ((mstr (make-expr (+ 1 (random 4)))))
+	  (let* ((str5 (string-append "(let () (if (begin " mstr " ) (begin " nstr ") (begin " str ")))"))
+		 (str6 (string-append "(let () (define (func) " str5 ") (define (hi) (func)) (hi))"))
+		 (str7 (string-append "(let () (cond ((begin " mstr ") " nstr ") (else " str ")))"))
+		 (str8 (string-append "(let () (define (func) " str7 ") (define (hi) (func)) (hi))")))
+	    (let ((val5 (eval-it str5))
+		  (val6 (eval-it str6))
+		  (val7 (eval-it str7))
+		  (val8 (eval-it str8)))
+	      (same-type? val5 val6 val7 val8 nstr str5 str6 str7 str8))))
+|#
+	)
 
     (define dots (vector "." "-" "+" "-"))
     (define (test-it)
@@ -831,18 +1095,12 @@
 	  (if (= n 4) (set! n 0))
 	  (format *stderr* "~A" (vector-ref dots n)))
 
-	(try-both (make-expr (+ 1 (random 8)))) ; min 1 here not 0
-	(set! __var__ #f)
-#|	
-	(catch #t
-	  (lambda ()
-	    (try-both (make-expr (+ 1 (random 9))))) ; min 1 here not 0
-	  (lambda (type info)
-	    (format *stderr* "~A: ~S ~S ~S~%" type (apply format #f info) ostr (owlet))))
-|#
+	(try-both (make-expr (+ 1 (random 6)))) ; min 1 here not 0
+	(set! __var__ ((lambda args (car args)) (catch #t (lambda () (eval-string (get-arg))) (lambda () #f))))
+	(if (iterator? __var__) (set! __var__ #f))
+	(set! __old_var__ __var__)
 	))
 
     (test-it)))
 
-;;; arity/signature checks? -- need access to returning caller
 
